import pino from 'pino'
import fs from 'fs'
import path from 'path'

// Array of log types
const LOG_TYPES = ['http', 'game', 'gameMaster', 'app', 'queue'] as const
type LogType = (typeof LOG_TYPES)[number]

const createLogger = (name: LogType) =>
  pino({
    level: process.env.LOG_LEVEL || 'debug',
    transport: {
      target: 'pino/file',
      options: {
        destination: `./logs/${name}.log`,
        mkdir: true
      }
    },
    formatters: {
      level: (label) => {
        return { level: label.toUpperCase() }
      }
    },
    timestamp: pino.stdTimeFunctions.isoTime,
    base: null
  })

// Create logger instances
const loggers = Object.fromEntries(LOG_TYPES.map((type) => [type, createLogger(type)])) as Record<LogType, ReturnType<typeof createLogger>>

const watchLogs = () => {
  LOG_TYPES.forEach((type) => {
    const logFile = path.join(__dirname, '../../logs', `${type}.log`)

    fs.watchFile(logFile, (curr, prev) => {
      if (curr.size > prev.size) {
        const stream = fs.createReadStream(logFile, { start: prev.size, end: curr.size })
        stream.on('data', (chunk) => {
          console.log(`[${type}]\n${chunk.toString()}`)
        })
      }
    })
  })
}

export const { http: httpLogger, game: gameLogger, gameMaster: gameMasterLogger, app: appLogger, queue: queueLogger } = loggers

export { watchLogs }
